# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=line-too-long


from azure.mgmt.signalr.models import (
    CustomDomain,
    ResourceReference
)

from azure.mgmt.signalr._signal_rmanagement_client import (
    SignalRCustomDomainsOperations
)


def custom_domain_create(client: SignalRCustomDomainsOperations, resource_group_name, signalr_name, name, domain_name, certificate_resource_id):
    resource_reference = ResourceReference(id=certificate_resource_id)
    custom_domain = CustomDomain(domain_name=domain_name, custom_certificate=resource_reference)

    return client.begin_create_or_update(resource_group_name, signalr_name, name, custom_domain)


def custom_domain_delete(client: SignalRCustomDomainsOperations, resource_group_name, signalr_name, name):
    return client.begin_delete(resource_group_name, signalr_name, name)


def custom_domain_show(client: SignalRCustomDomainsOperations, resource_group_name, signalr_name, name):
    return client.get(resource_group_name, signalr_name, name)


def custom_domain_list(client: SignalRCustomDomainsOperations, resource_group_name, signalr_name):
    return client.list(resource_group_name, signalr_name)


def get_custom_domain(client: SignalRCustomDomainsOperations, resource_group_name, signalr_name, name):
    return client.get(resource_group_name, signalr_name, name)


def set_custom_domain(client: SignalRCustomDomainsOperations, resource_group_name, signalr_name, name, parameters):
    return client.begin_create_or_update(resource_group_name, signalr_name, name, parameters)


def update(instance: CustomDomain, domain_name=None, certificate_resource_id=None):
    if domain_name is not None:
        instance.domain_name = domain_name
    if certificate_resource_id is not None:
        instance.custom_certificate = ResourceReference(id=certificate_resource_id)
    return instance



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from base64 import b64encode, b64decode
from hashlib import sha256
from hmac import HMAC
from time import time
try:
    from urllib import (urlencode, quote)
except ImportError:
    from urllib.parse import (urlencode, quote)  # pylint: disable=import-error
from msrest.authentication import Authentication


class SasTokenAuthentication(Authentication):
    """
    Shared Access Signature authorization for Azure IoT Hub.

    :param str uri: Uri of target resource.
    :param str shared_access_policy_name: Name of shared access policy.
    :param str shared_access_key: Shared access key.
    :param int expiry: Expiry of the token to be generated.
    Input should be seconds since the epoch, in UTC. Default is an hour later from now.
    """
    def __init__(self, uri, shared_access_policy_name, shared_access_key, expiry=None):

        self.uri = quote(uri.lower(), safe='').lower()
        self.policy = shared_access_policy_name
        self.key = shared_access_key
        if expiry is None:
            self.expiry = time() + 3600  # Default expiry is an hour later
        else:
            self.expiry = expiry

    def signed_session(self, session=None):
        """Create requests session with SAS auth headers.

        :rtype: requests.Session.
        """
        session = session or super(SasTokenAuthentication, self).signed_session()
        session.headers['Authorization'] = self.generate_sas_token()
        return session

    def generate_sas_token(self):
        encoded_uri = quote(self.uri, safe='').lower()
        ttl = int(self.expiry)
        sign_key = '%s\n%d' % (encoded_uri, ttl)
        signature = b64encode(HMAC(b64decode(self.key), sign_key.encode('utf-8'), sha256).digest())

        return 'SharedAccessSignature ' + urlencode({
            'sr': self.uri,
            'sig': signature,
            'se': str(ttl),
            'skn': self.policy
        })



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands import CliCommandType


def load_command_table(self, _):

    configure_custom = CliCommandType(operations_tmpl='azure.cli.command_modules.configure.custom#{}')

    with self.command_group('', configure_custom) as g:
        g.command('configure', 'handle_configure')

    with self.command_group('cache', configure_custom) as g:
        g.command('list', 'list_cache_contents')
        g.command('show', 'show_cache_contents')  # pylint: disable=show-command
        g.command('delete', 'delete_cache_contents')
        g.command('purge', 'purge_cache_contents')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.log import get_logger
from knack.util import CLIError

from azure.cli.core.extension import (
    ExtensionNotInstalledException, get_extension_modname, get_extension)
from azure.cli.core.extension.operations import (
    reload_extension, update_extension, add_extension, add_extension_to_path, check_version_compatibility)

logger = get_logger(__name__)
INTERACTIVE_EXTENSION_NAME = 'interactive'


def start_shell(cmd, update=None, style=None):
    from importlib import import_module

    try:
        get_extension(INTERACTIVE_EXTENSION_NAME)

        if update:
            logger.warning("Updating the Interactive extension to the latest available...")
            update_extension(cmd, INTERACTIVE_EXTENSION_NAME)
            reload_extension(INTERACTIVE_EXTENSION_NAME)
    except ExtensionNotInstalledException:
        logger.warning("Installing the Interactive extension...")
        add_extension(cmd, extension_name=INTERACTIVE_EXTENSION_NAME)

    ext = get_extension(INTERACTIVE_EXTENSION_NAME)
    try:
        check_version_compatibility(ext.get_metadata())
    except CLIError:
        raise CLIError('Run `az interactive --update` and try again.')

    add_extension_to_path(INTERACTIVE_EXTENSION_NAME, ext_dir=ext.path)
    interactive_module = get_extension_modname(ext_name=INTERACTIVE_EXTENSION_NAME, ext_dir=ext.path)
    azext_interactive = import_module(interactive_module)
    azext_interactive.start_shell(cmd, style=style)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=unused-argument
from azure.cli.core.util import sdk_no_wait
from azure.mgmt.synapse.models import UpdateIntegrationRuntimeNodeRequest


def update(cmd, client, resource_group_name, workspace_name, integration_runtime_name, auto_update,
           update_delay_offset, node_name, no_wait=False):
    update_integration_runtime_node_request = UpdateIntegrationRuntimeNodeRequest(
        auto_update=auto_update,
        update_delay_offset=update_delay_offset
    )
    return sdk_no_wait(no_wait, client.update, resource_group_name, workspace_name, integration_runtime_name,
                       node_name, update_integration_runtime_node_request)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=redefined-builtin

from azure.mgmt.media.models import (MediaService, MediaServiceUpdate, MediaServiceIdentity)
from azure.cli.core.azclierror import ValidationError


def assign_identity(client, resource_group_name, account_name, system_assigned=False, user_assigned=None):
    if system_assigned is False and user_assigned is None:
        error_msg = 'Please specify either system assigned identity or a user assigned identity'
        raise ValidationError(error_msg)

    account_info = client.get(resource_group_name,
                              account_name) if resource_group_name else client.get_by_subscription(account_name)

    current_identity_types = account_info.identity.type.split(',')
    if 'None' in current_identity_types:
        current_identity_types = []

    media_service = MediaServiceUpdate(identity=MediaServiceIdentity(type=''))

    if system_assigned and 'SystemAssigned' not in current_identity_types:
        current_identity_types.append('SystemAssigned')
    user_id_dict = {}
    if user_assigned:
        if 'UserAssigned' not in current_identity_types:
            current_identity_types.append('UserAssigned')
        for user_id in user_assigned:
            user_id_dict[user_id] = {}
        media_service.identity.user_assigned_identities = user_id_dict

    identity_type = ','.join(current_identity_types)
    media_service.identity.type = identity_type

    return client.update(resource_group_name, account_name, media_service)


def remove_identity(client, resource_group_name, account_name, system_assigned=False, user_assigned=None):
    if (system_assigned is False and user_assigned is None):
        error_msg = 'Please specify either system assigned identity or a user assigned identity'
        raise ValidationError(error_msg)

    account_info = client.get(resource_group_name,
                              account_name) if resource_group_name else client.get_by_subscription(account_name)

    current_identity_types = account_info.identity.type.split(',')

    media_service = MediaService(name=account_info.name, location=account_info.location,
                                 key_delivery=account_info.key_delivery,
                                 identity=MediaServiceIdentity(
                                     type='',
                                     user_assigned_identities=account_info.identity.user_assigned_identities),
                                 encryption=account_info.encryption, storage_accounts=account_info.storage_accounts,
                                 storage_authentication=account_info.storage_authentication,
                                 public_network_access=account_info.public_network_access)

    if system_assigned and 'SystemAssigned' in current_identity_types:
        current_identity_types.remove('SystemAssigned')
    user_id_dict = {}
    if user_assigned:
        for user_id in user_assigned:
            user_id_dict[user_id] = None
        media_service.identity.user_assigned_identities = user_id_dict

    identity_type = ','.join(current_identity_types) if current_identity_types else 'None'
    media_service.identity.type = identity_type

    if user_assigned and identity_type != 'None':
        return client.update(resource_group_name, account_name, media_service)

    return client.create_or_update(resource_group_name, account_name, media_service)


def show_identity(client, resource_group_name, account_name):
    account_info = client.get(resource_group_name,
                              account_name) if resource_group_name else client.get_by_subscription(account_name)
    return account_info.identity



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands import CliCommandType
from azure.cli.core.commands.arm import deployment_validate_table_format, handle_template_based_exception

from ._format import (
    transform_sqlvm_group_output,
    transform_sqlvm_group_list,
    transform_sqlvm_output,
    transform_sqlvm_list,
    transform_aglistener_output,
    transform_aglistener_list
)

from ._util import (
    get_sqlvirtualmachine_availability_group_listeners_operations,
    get_sqlvirtualmachine_sql_virtual_machine_groups_operations,
    get_sqlvirtualmachine_sql_virtual_machines_operations,
)


# pylint: disable=too-many-statements,line-too-long,too-many-locals
def load_command_table(self, _):

    ###############################################
    #            sql virtual machine              #
    ###############################################

    sqlvm_vm_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sqlvirtualmachine.operations#SqlVirtualMachinesOperations.{}',
        client_factory=get_sqlvirtualmachine_sql_virtual_machines_operations
    )

    with self.command_group('sql vm',
                            sqlvm_vm_operations,
                            client_factory=get_sqlvirtualmachine_sql_virtual_machines_operations) as g:
        g.generic_update_command('update', custom_func_name='sqlvm_update', setter_name='begin_create_or_update', transform=transform_sqlvm_output)
        g.show_command('show', 'get', transform=transform_sqlvm_output)
        g.custom_command('list', 'sqlvm_list', transform=transform_sqlvm_list)
        g.custom_command('add-to-group', 'sqlvm_add_to_group', transform=transform_sqlvm_output)
        g.custom_command('remove-from-group', 'sqlvm_remove_from_group', transform=transform_sqlvm_output)
        g.command('delete', 'begin_delete', confirmation=True)
        g.custom_command('create', 'sqlvm_create', transform=transform_sqlvm_output, table_transformer=deployment_validate_table_format, exception_handler=handle_template_based_exception)
        g.command('start-assessment', 'begin_start_assessment')

    ###############################################
    #      sql virtual machine groups             #
    ###############################################

    sqlvm_group_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sqlvirtualmachine.operations#SqlVirtualMachineGroupsOperations.{}',
        client_factory=get_sqlvirtualmachine_sql_virtual_machine_groups_operations
    )

    with self.command_group('sql vm group',
                            sqlvm_group_operations,
                            client_factory=get_sqlvirtualmachine_sql_virtual_machine_groups_operations) as g:
        g.generic_update_command('update', custom_func_name='sqlvm_group_update', setter_name='begin_create_or_update', transform=transform_sqlvm_group_output)
        g.show_command('show', 'get', transform=transform_sqlvm_group_output)
        g.custom_command('list', 'sqlvm_group_list', transform=transform_sqlvm_group_list)
        g.command('delete', 'begin_delete', confirmation=True)
        g.custom_command('create', 'sqlvm_group_create', transform=transform_sqlvm_group_output, table_transformer=deployment_validate_table_format, exception_handler=handle_template_based_exception)

    ###############################################
    #      availability group listener            #
    ###############################################

    sqlvm_agl_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sqlvirtualmachine.operations#AvailabilityGroupListenersOperations.{}',
        client_factory=get_sqlvirtualmachine_availability_group_listeners_operations
    )

    with self.command_group('sql vm group ag-listener',
                            sqlvm_agl_operations,
                            client_factory=get_sqlvirtualmachine_availability_group_listeners_operations) as g:
        g.generic_update_command('update', custom_func_name='aglistener_update', setter_name='begin_create_or_update', transform=transform_aglistener_output)
        g.show_command('show', 'get', transform=transform_aglistener_output)
        g.command('list', 'list_by_group', transform=transform_aglistener_list)
        g.command('delete', 'begin_delete', confirmation=True)
        g.custom_command('create', 'sqlvm_aglistener_create', transform=transform_aglistener_output, table_transformer=deployment_validate_table_format, exception_handler=handle_template_based_exception)

    with self.command_group('sql vm', is_preview=True):
        pass



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------
# pylint: disable=too-many-lines
# pylint: disable=too-many-statements

from azure.cli.core.commands.parameters import resource_group_name_type


def load_arguments(self, _):

    with self.argument_context('sshkey generate') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('ssh_public_key_name', options_list=['--name', '-n', '--ssh-public-key-name'], type=str, help='The '
                   'name of the SSH public key.', id_part='name')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands import LongRunningOperation


def _last_segment(resource_id):
    return resource_id.split('/')[-1] if resource_id else None


_units = [(1024 * 1024 * 1024 * 1024, 'TB'),
          (1024 * 1024 * 1024, 'GB'),
          (1024 * 1024, 'MB'),
          (1024, 'kB'),
          (1, 'B')]


def _bytes_to_friendly_string(b):
    '''
    Formats the specified integer count of bytes as a friendly string
    with units, e.g. 1024 -> "1kB"
    '''

    # Find the largest unit that evenly divides the input
    unit = next(u for u in _units if (b % u[0]) == 0)

    # Format the value with the chosen unit
    return str((b // unit[0])) + unit[1]


class LongRunningOperationResultTransform(LongRunningOperation):  # pylint: disable=too-few-public-methods
    '''
    Long-running operation poller that also transforms the json response.
    '''
    def __init__(self, cli_ctx, transform_func):
        super(LongRunningOperationResultTransform, self).__init__(cli_ctx)
        self._transform_func = transform_func

    def __call__(self, result):
        '''
        Function call operator which will do polling (if necessary)
        and then transforms the result.
        '''

        if result is None:
            return None

        from azure.cli.core.util import poller_classes
        if isinstance(result, poller_classes()):
            # Poll for long-running operation result result by calling base class
            result = super(LongRunningOperationResultTransform, self).__call__(result)

        # Apply transform function
        return self._transform_func(result)


def _apply_format(result, format_group):
    '''
    Applies the specified format_group function to the single result or list of results.
    '''

    # Optionally grab internal 'value' array
    if 'value' in result and isinstance(result['value'], list):
        result = result['value']

    # Get list of results, or make singleton list from single list
    obj_list = result if isinstance(result, list) else [result]

    # Apply format function to list
    return [format_group(item) for item in obj_list]

###############################################
#                sql instance-pool            #
###############################################


def instance_pool_table_format(result):
    '''
    Formats an instance pool or list of instance pools as summary results for display with "-o table".
    '''
    def _instance_pool_table_format(result):
        '''
        Formats an instance pool or list of instance pools as summary results for display with "-o table".
        '''
        from collections import OrderedDict
        sku = result['sku']
        return OrderedDict([
            ('name', result['name']),
            ('resourceGroup', result['resourceGroup']),
            ('location', result['location']),
            ('Capacity', result['vCores']),
            ('SKU Family', sku['family']),
            ('SKU Tier', sku['tier']),
            ('Tags', str(result['tags']) if result['tags'] else '')
        ])

    return _apply_format(result, _instance_pool_table_format)

###############################################
#                sql server                   #
###############################################


def usage_table_format(result):
    '''
    Formats a usage or list of usage as summary results for display with "-o table".
    '''

    def _usage_table_format(result):
        '''
        Formats a usage as summary results for display with "-o table".
        '''
        from collections import OrderedDict

        return OrderedDict([
            ('name', result['name']),
            ('displayName', result['displayName']),
            ('currentValue', result['currentValue']),
            ('limit', result['limit']),
            ('unit', result['unit'])
        ])

    return _apply_format(result, _usage_table_format)


###############################################
#                sql db                       #
###############################################


#####
#           sql db transformers for json
#####


def db_list_transform(results):
    '''
    Transforms the json response for a list of databases.
    '''

    return [db_transform(r) for r in results]


def db_transform(result):
    '''
    Transforms the json response for a database.
    '''

    # Add properties in order to improve backwards compatibility with api-version 2014-04-01
    result.edition = result.sku.tier
    result.elastic_pool_name = _last_segment(result.elastic_pool_id)

    # rename ledger setting name
    if hasattr(result, 'is_ledger_on'):
        result.ledger_on = result.is_ledger_on
        del result.is_ledger_on

    if hasattr(result, 'storage_account_type'):
        result.backupStorageRedundancy = _get_external_backup_storage_redundancy(result.storage_account_type)
        del result.storage_account_type

    return result


def _get_external_backup_storage_redundancy(self):
    return {
        'lrs': 'Local',
        'grs': 'Geo',
        'zrs': 'Zone'
    }.get(self.lower(), 'Invalid')


#####
#           sql db table formatters
#####


def db_table_format(result):
    '''
    Formats a database or list of databases as summary results for display with "-o table".
    '''

    def _db_table_format(result):
        '''
        Formats a single database as summary results for display with "-o table".
        '''
        from collections import OrderedDict

        return OrderedDict([
            ('name', result['name']),
            ('tier', result['sku']['tier']),
            ('family', result['sku']['family'] or ' '),
            ('capacity', result['sku']['capacity'] or ' '),
            ('maxSize', _bytes_to_friendly_string(result['maxSizeBytes'])),
            ('elasticPool', _last_segment(result['elasticPoolId']) or ' '),
        ])

    return _apply_format(result, _db_table_format)


def db_edition_table_format(editions):
    '''
    Formats a list of database editions as summary results for display with "-o table".
    '''

    return list(_db_edition_table_format(editions))


def _db_edition_table_format(editions):
    '''
    Formats a database edition as summary results for display with "-o table".
    '''
    from .custom import is_available
    from collections import OrderedDict

    for e in editions:
        for slo in e['supportedServiceLevelObjectives']:
            sku = slo['sku']
            yield OrderedDict([
                ('serviceObjective', slo['name']),
                ('sku', sku['name']),
                ('edition', e['name']),
                # Dummy ' ' value ensures that value is not skipped, which
                # would cause the column to not show up in the correct order
                ('family', sku['family'] or ' '),
                ('capacity', sku['capacity']),
                ('unit', slo['performanceLevel']['unit']),
                ('available', is_available(slo['status'])),
            ])


###############################################
#                sql elastic-pool             #
###############################################


#####
#           sql elastic-pool transformers for json
#####


def elastic_pool_list_transform(results):
    return [elastic_pool_transform(r) for r in results]


def elastic_pool_transform(result):
    '''
    Transforms the json response for an elastic pool.
    '''

    # Add properties in order to improve backwards compatibility with api-version 2014-04-01
    result.edition = result.sku.tier
    result.storageMb = result.max_size_bytes / 1024 / 1024

    is_dtu = result.sku.tier in (
        'Basic',
        'Standard',
        'Premium')

    result.dtu = result.sku.capacity if is_dtu else None
    result.database_dtu_min = int(result.per_database_settings.min_capacity) if is_dtu else None
    result.database_dtu_max = int(result.per_database_settings.max_capacity) if is_dtu else None

    return result


#####
#           sql elastic-pool table formatters
#####


def elastic_pool_table_format(result):
    '''
    Formats a single or list of elastic pools as summary results for display with "-o table".
    '''

    def _elastic_pool_table_format(result):
        '''
        Formats a single elastic pool as summary results for display with "-o table".
        '''
        from collections import OrderedDict

        return OrderedDict([
            ('name', result['name']),
            ('tier', result['sku']['tier']),
            ('family', result['sku']['family'] or ' '),
            ('capacity', result['sku']['capacity'] or ' '),
            ('maxSize', _bytes_to_friendly_string(result['maxSizeBytes']))
        ])

    return _apply_format(result, _elastic_pool_table_format)


def elastic_pool_edition_table_format(editions):
    '''
    Formats a list of elastic pool editions as summary results for display with "-o table".
    '''

    return list(_elastic_pool_edition_table_format(editions))


def _elastic_pool_edition_table_format(editions):
    '''
    Formats an elastic pool editions as summary results for display with "-o table".
    '''
    from collections import OrderedDict
    from .custom import is_available

    for e in editions:
        for slo in e['supportedElasticPoolPerformanceLevels']:
            sku = slo['sku']
            yield OrderedDict([
                ('sku', sku['name']),
                ('edition', e['name']),
                # Dummy ' ' value ensures that value is not skipped, which
                # would cause the column to not show up in the correct order
                ('family', sku['family'] or ' '),
                ('capacity', sku['capacity']),
                ('unit', slo['performanceLevel']['unit']),
                ('available', is_available(slo['status'])),
            ])

###############################################
#                sql server                   #
###############################################


def server_table_format(result):
    '''
    Formats a server or list of servers as summary results for display with "-o table".
    '''

    def _server_table_format(result):
        '''
        Formats a server as summary results for display with "-o table".
        '''
        from collections import OrderedDict

        return OrderedDict([
            ('name', result['name']),
            ('resourceGroup', result['resourceGroup']),
            ('location', result['location']),
            ('administratorLogin', result['administratorLogin'])
        ])

    return _apply_format(result, _server_table_format)


###############################################
#                sql server firewall-rule     #
###############################################


def firewall_rule_table_format(result):
    '''
    Formats a single or list of server firewall rules as summary results for display with "-o table".
    '''

    def _firewall_rule_table_format(result):
        '''
        Formats a server firewall rule as summary results for display with "-o table".
        '''
        from collections import OrderedDict

        return OrderedDict([
            ('name', result['name']),
            ('startIpAddress', result['startIpAddress']),
            ('endIpAddress', result['endIpAddress'])
        ])

    return _apply_format(result, _firewall_rule_table_format)


########################################################
#            sql server outbound-firewall-rule         #
########################################################


def outbound_firewall_rule_table_format(result):
    '''
    Formats a single or list of server outbound firewall rules as summary results for display with "-o table".
    '''

    def _outbound_firewall_rule_table_format(result):
        '''
        Formats a server outbound firewall rule as summary results for display with "-o table".
        '''
        from collections import OrderedDict

        return OrderedDict([
            ('resourceGroupName', result['resourceGroupName']),
            ('serverName', result['serverName']),
            ('outboundRuleFqdn', result['outboundRuleFqdn'])
        ])

    return _apply_format(result, _outbound_firewall_rule_table_format)
